@@ -3,8 +3,8 @@
   Program:   CMake - Cross-Platform Makefile Generator
   Module:    $RCSfile: cmCPackSTGZGenerator.h,v $
   Language:  C++
-  Date:      $Date: 2006/04/18 12:25:24 $
-  Version:   $Revision: 1.7 $
+  Date:      $Date: 2007-02-02 19:40:26 $
+  Version:   $Revision: 1.8 $
 
   Copyright (c) 2002 Kitware, Inc. All rights reserved.
   See Copyright.txt or http://www.cmake.org/HTML/Copyright.html for details.
@@ -41,7 +41,7 @@ protected:
     const std::vector<std::string>& files);
   virtual int InitializeInternal();
   int GenerateHeader(std::ostream* os);
-  virtual const char* GetOutputExtension() { return "sh"; }
+  virtual const char* GetOutputExtension() { return ".sh"; }
 };
 
 #endif
