@@ -3,8 +3,8 @@
   Program:   CMake - Cross-Platform Makefile Generator
   Module:    $RCSfile: cmCPackZIPGenerator.h,v $
   Language:  C++
-  Date:      $Date: 2007/02/05 18:21:32 $
-  Version:   $Revision: 1.2.2.1 $
+  Date:      $Date: 2007-11-05 21:55:45 $
+  Version:   $Revision: 1.6 $
 
   Copyright (c) 2002 Kitware, Inc. All rights reserved.
   See Copyright.txt or http://www.cmake.org/HTML/Copyright.html for details.
@@ -18,18 +18,18 @@
 #ifndef cmCPackZIPGenerator_h
 #define cmCPackZIPGenerator_h
 
-#include "cmCPackGenericGenerator.h"
+#include "cmCPackGenerator.h"
 
 class cmCPackZIPGeneratorForward;
 
 /** \class cmCPackZIPGenerator
  * \brief A generator for ZIP files
  */
-class cmCPackZIPGenerator : public cmCPackGenericGenerator
+class cmCPackZIPGenerator : public cmCPackGenerator
 {
 public:
   friend class cmCPackZIPGeneratorForward;
-  cmCPackTypeMacro(cmCPackZIPGenerator, cmCPackGenericGenerator);
+  cmCPackTypeMacro(cmCPackZIPGenerator, cmCPackGenerator);
 
   /**
    * Construct generator
@@ -37,13 +37,6 @@ public:
   cmCPackZIPGenerator();
   virtual ~cmCPackZIPGenerator();
 
-  enum ZipStyles
-    {
-    StyleUnkown,
-    StyleWinZip,
-    StyleUnixZip
-    };
-
 protected:
   virtual int InitializeInternal();
   int CompressFiles(const char* outFileName, const char* toplevel,
