@@ -3,8 +3,8 @@
   Program:   CMake - Cross-Platform Makefile Generator
   Module:    $RCSfile: cmInstallScriptGenerator.h,v $
   Language:  C++
-  Date:      $Date: 2006/04/13 02:04:50 $
-  Version:   $Revision: 1.2 $
+  Date:      $Date: 2007-10-15 11:08:10 $
+  Version:   $Revision: 1.3 $
 
   Copyright (c) 2002 Kitware, Inc., Insight Consortium.  All rights reserved.
   See Copyright.txt or http://www.cmake.org/HTML/Copyright.html for details.
@@ -25,7 +25,8 @@
 class cmInstallScriptGenerator: public cmInstallGenerator
 {
 public:
-  cmInstallScriptGenerator(const char* script, bool code = false);
+  cmInstallScriptGenerator(const char* script, bool code,
+    const char* component);
   virtual ~cmInstallScriptGenerator();
 
 protected:
